<?php defined('ABSPATH') OR exit;
/*
Plugin Name: Import from YML
Description: Import products from YML to WooCommerce
Tags: yml, yandex, import, export, woocommerce
Author: Maxim Glazunov
Author URI: https://icopydoc.ru
License: GPLv2
Version: 1.3.0
Text Domain: import-from-yml
Domain Path: /languages/
WC requires at least: 3.0.0
WC tested up to: 5.6.0
*/
/*	Copyright YEAR PLUGIN_AUTHOR_NAME (email : djdiplomat@yandex.ru)
 
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as
	published by the Free Software Foundation.
 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/
require_once plugin_dir_path(__FILE__).'/functions.php'; // Подключаем файл функций
require_once plugin_dir_path(__FILE__).'/offer.php';
require_once plugin_dir_path(__FILE__).'/product_add.php';
require_once plugin_dir_path(__FILE__).'/product_upd.php';
register_activation_hook(__FILE__, array('ImportProductsYMLtoWooCommerce', 'on_activation'));
register_deactivation_hook(__FILE__, array('ImportProductsYMLtoWooCommerce', 'on_deactivation'));
register_uninstall_hook(__FILE__, array('ImportProductsYMLtoWooCommerce', 'on_uninstall'));
add_action('plugins_loaded', array('ImportProductsYMLtoWooCommerce', 'init'));
add_action('plugins_loaded', 'ipytw_load_plugin_textdomain'); // load translation
function ipytw_load_plugin_textdomain() {
 load_plugin_textdomain('ipytw', false, dirname(plugin_basename(__FILE__)).'/languages/');
}
class ImportProductsYMLtoWooCommerce {
 protected static $instance;
 public static function init() {
	is_null(self::$instance) AND self::$instance = new self;
	return self::$instance;
 }

 public function __construct() {
	// ipytw_DIR contains /home/p135/www/site.ru/wp-content/plugins/myplagin/
	define('ipytw_DIR', plugin_dir_path(__FILE__)); 
	// ipytw_URL contains http://site.ru/wp-content/plugins/myplagin/
	define('ipytw_URL', plugin_dir_url(__FILE__));
	// ipytw_UPLOAD_DIR contains /home/p256/www/site.ru/wp-content/uploads
	$upload_dir = (object)wp_get_upload_dir();
	define('ipytw_UPLOAD_DIR', $upload_dir->basedir);
	// ipytw_UPLOAD_DIR contains /home/p256/www/site.ru/wp-content/uploads/import-from-yml
	$name_dir = $upload_dir->basedir."/import-from-yml"; 
	define('ipytw_NAME_DIR', $name_dir);
	$ipytw_keeplogs = ipytw_optionGET('ipytw_keeplogs');
	define('ipytw_KEEPLOGS', $ipytw_keeplogs);
	define('ipytw_VER', '1.3.0');
	$ipytw_version = ipytw_optionGET('ipytw_version');
  	if ($ipytw_version !== ipytw_VER) {ipytw_set_new_options();} // автообновим настройки, если нужно

	add_action('admin_menu', array($this, 'add_admin_menu'));
	add_filter('upload_mimes', array($this, 'ipytw_add_mime_types'), 9999); // чутка позже остальных

	add_filter('cron_schedules', array($this, 'cron_add_fifty_sec'));
	add_filter('cron_schedules', array($this, 'cron_add_five_min'));	
	add_filter('cron_schedules', array($this, 'cron_add_six_hours'));
	add_filter('cron_schedules', array($this, 'cron_add_every_two_days'));

	add_action('ipytw_cron_period', array($this, 'ipytw_do_this_event'), 10, 1);
	add_action('ipytw_cron_sborki', array($this, 'ipytw_do_this_fifty_sec'), 10, 1);
	
	add_action('admin_notices', array($this, 'ipytw_admin_notices_function'));

	/* Регаем стили только для страницы настроек плагина */
	add_action('admin_init', function() {
		wp_register_style('ipytw-admin-css', plugins_url('css/ipytw_style.css', __FILE__));
	}, 9999);

	add_filter('plugin_action_links', array($this, 'ipytw_plugin_action_links'), 10, 2 );
 }

 public static function ipytw_plugin_action_links($actions, $plugin_file) {
	if (false === strpos($plugin_file, basename(__FILE__))) { // проверка, что у нас текущий плагин
		return $actions;
	}
	$settings_link = '<a style="color: green; font-weight: 700;" href="/wp-admin/admin.php?page=ipytwextensions">'. __('More features', 'ipytw').'</a>';
	array_unshift($actions, $settings_link); 	
	$settings_link = '<a href="/wp-admin/admin.php?page=ipytwexport">'. __('Settings', 'ipytw').'</a>';
	array_unshift($actions, $settings_link); 
	return $actions; 
 }

 public static function ipytw_admin_css_func() {
	/* Ставим css-файл в очередь на вывод */
	wp_enqueue_style('ipytw-admin-css');
 } 

 public static function ipytw_admin_head_css_func() {
	/* печатаем css в шапке админки */
	print '<style>/* Import from YML */
		.metabox-holder .postbox-container .empty-container {height: auto !important;}
		.icp_img1 {background-image: url('. ipytw_URL .'img/sl1.jpg);}
		.icp_img2 {background-image: url('. ipytw_URL .'img/sl2.jpg);}
		.icp_img3 {background-image: url('. ipytw_URL .'img/sl3.jpg);}
		.icp_img4 {background-image: url('. ipytw_URL .'img/sl4.jpg);}
		.icp_img5 {background-image: url('. ipytw_URL .'img/sl5.jpg);}
		.icp_img6 {background-image: url('. ipytw_URL .'img/sl6.jpg);}
		.icp_img7 {background-image: url('. ipytw_URL .'img/sl7.jpg);}
	</style>';
 }  
 
 // Срабатывает при активации плагина (вызывается единожды)
 public static function on_activation() {
	$upload_dir = (object)wp_get_upload_dir();
	$name_dir = $upload_dir->basedir.'/import-from-yml';
	if (!is_dir($name_dir)) {
	 if (!mkdir($name_dir)) {
		error_log('ERROR: Ошибка создания папки '.$name_dir.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
	 }
	}

	$ipytw_settings_arr = array(
		'1' => ipytw_set_default_feed_settings_arr()
	);

	$ipytw_registered_feeds_arr = array(
		0 => array('last_id' => '1'),
		1 => array('id' => '1')
	);

	if (is_multisite()) {
		add_blog_option(get_current_blog_id(), 'ipytw_keeplogs', '0');
		add_blog_option(get_current_blog_id(), 'ipytw_enable_backend_debug', '0');
		add_blog_option(get_current_blog_id(), 'ipytw_version', '1.3.0');
		add_blog_option(get_current_blog_id(), 'ipytw_disable_notices', '0');
		add_blog_option(get_current_blog_id(), 'ipytw_enable_five_min', '0');
		add_blog_option(get_current_blog_id(), 'ipytw_errors', '');
		add_blog_option(get_current_blog_id(), 'ipytw_settings_arr', $ipytw_settings_arr);
		add_blog_option(get_current_blog_id(), 'ipytw_status_sborki1', '-1');
		add_blog_option(get_current_blog_id(), 'ipytw_last_element1', '-1');
		add_blog_option(get_current_blog_id(), 'ipytw_registered_feeds_arr', $ipytw_registered_feeds_arr);
	} else {
		add_option('ipytw_keeplogs', '0');
		add_option('ipytw_enable_backend_debug', '0');
		add_option('ipytw_version', '1.3.0', '', 'no'); // без автозагрузки
		add_option('ipytw_disable_notices', '0', '', 'no');
		add_option('ipytw_enable_five_min', '0', '', 'no');
		add_option('ipytw_errors', '', '', 'no');
		add_option('ipytw_settings_arr', $ipytw_settings_arr);
		add_option('ipytw_status_sborki1', '-1');
		add_option('ipytw_last_element1', '-1');
		add_option('ipytw_registered_feeds_arr', $ipytw_registered_feeds_arr);
	}
 }
 
 // Срабатывает при отключении плагина (вызывается единожды)
 public static function on_deactivation() {
	$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
	for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
		$feedId = $ipytw_registered_feeds_arr[$i]['id'];
		wp_clear_scheduled_hook('ipytw_cron_period', array($feedId)); // отключаем крон
		wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId)); // отключаем крон
	}	
 } 
 
 // Срабатывает при удалении плагина (вызывается единожды)
 public static function on_uninstall() {
	if (is_multisite()) {
		$ipytw_registered_feeds_arr = get_blog_option(get_current_blog_id(), 'ipytw_registered_feeds_arr');
		for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
			$feedId = $ipytw_registered_feeds_arr[$i]['id'];
			if ($feedId === '0' || $feedId === 0) {$feedId = '';} // позже можно будет удалить
			delete_blog_option(get_current_blog_id(), 'ipytw_status_sborki'.$feedId);
			delete_blog_option(get_current_blog_id(), 'ipytw_last_element'.$feedId);
		}
		delete_blog_option(get_current_blog_id(), 'ipytw_keeplogs');
		delete_blog_option(get_current_blog_id(), 'ipytw_enable_backend_debug');
		delete_blog_option(get_current_blog_id(), 'ipytw_version');
		delete_blog_option(get_current_blog_id(), 'ipytw_disable_notices');
		delete_blog_option(get_current_blog_id(), 'ipytw_enable_five_min');
		delete_blog_option(get_current_blog_id(), 'ipytw_errors');
		delete_blog_option(get_current_blog_id(), 'ipytw_settings_arr');
		delete_blog_option(get_current_blog_id(), 'ipytw_registered_feeds_arr');		
	} else {
		$ipytw_registered_feeds_arr = get_option('ipytw_registered_feeds_arr');
		for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
			$feedId = $ipytw_registered_feeds_arr[$i]['id'];
			if ($feedId === '0' || $feedId === 0) {$feedId = '';} // позже можно будет удалить
			delete_option('ipytw_status_sborki'.$feedId);
			delete_option('ipytw_last_element'.$feedId);
		}
		delete_option('ipytw_keeplogs');
		delete_option('ipytw_enable_backend_debug');
		delete_option('ipytw_version');
		delete_option('ipytw_disable_notices');
		delete_option('ipytw_enable_five_min');
		delete_option('ipytw_errors');
		delete_option('ipytw_settings_arr');
		delete_option('ipytw_registered_feeds_arr');
	}
 }

 // Добавляем пункты меню
 public function add_admin_menu() {
	$page_suffix = add_menu_page(null , __('Import from YML', 'ipytw'), 'manage_options', 'ipytwexport', 'ipytw_export_page', 'dashicons-redo', 51);
	require_once ipytw_DIR.'/export.php'; // Подключаем файл настроек
	// создаём хук, чтобы стили выводились только на странице настроек
	add_action('admin_print_styles-'. $page_suffix, array($this, 'ipytw_admin_css_func'));
 	add_action('admin_print_styles-'. $page_suffix, array($this, 'ipytw_admin_head_css_func'));

	if (!class_exists('WP_List_Table')) {
		require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
	}
	require_once plugin_dir_path(__FILE__).'/includes/class_wp_list_table.php';

	add_submenu_page('ipytwexport', __('Debug', 'ipytw'), __('Debug page', 'ipytw'), 'manage_options', 'ipytwdebug', 'ipytw_debug_page');
	require_once ipytw_DIR.'/debug.php';
	$page_subsuffix = add_submenu_page('ipytwexport', __('Add Extensions', 'ipytw'), __('Extensions', 'ipytw'), 'manage_options', 'ipytwextensions', 'ipytw_extensions_page');
	require_once ipytw_DIR.'/extensions.php';
	add_action('admin_print_styles-'. $page_subsuffix, array($this, 'ipytw_admin_css_func'));
 } 
 
 // Разрешим загрузку xml и csv файлов
 public function ipytw_add_mime_types($mimes) {
	$mimes ['csv'] = 'text/csv';
	$mimes ['xml'] = 'text/xml';
	$mimes ['yml'] = 'text/yml';
	return $mimes;
 } 

 /* добавляем интервалы крон в 50 секунд, 5 минут, 6 часов и 2 суток */
 public function cron_add_fifty_sec($schedules) {
	$schedules['fifty_sec'] = array(
		'interval' => 50,
		'display' => '50 sec'
	);
	return $schedules;
 }
 public function cron_add_five_min($schedules) {
	$schedules['five_min'] = array(
		'interval' => 300,
		'display' => '5 min'
	);
	return $schedules;
 } 
 public function cron_add_six_hours($schedules) {
	$schedules['six_hours'] = array(
		'interval' => 21600,
		'display' => '6 hours'
	);
	return $schedules;
 }
 public function cron_add_every_two_days($schedules) {
	$schedules['every_two_days'] = array(
		'interval' => 172800,
		'display' => __('Every two days', 'ipytw')
	);
	return $schedules;
 } 
 /* end добавляем интервалы крон в 50 секунд, 5 минут, 6 часов и 2 суток */ 

 /* функции крона */
 public function ipytw_do_this_event($feedId) {
	ipytw_error_log('FEED № '.$feedId.'; Крон ipytw_do_this_event включен. Делаем что-то каждый час; Файл: import-from-yml.php; Строка: '.__LINE__, 0);

	ipytw_optionUPD('ipytw_status_sborki', 1, $feedId);
	ipytw_optionUPD('ipytw_last_element', 0, $feedId); // выставляем в положения начала сборки

	wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId));

	// Возвращает nul/false. null когда планирование завершено. false в случае неудачи.
	$res = wp_schedule_event(time(), 'fifty_sec', 'ipytw_cron_sborki', array($feedId));
	if ($res === false) {
		ipytw_error_log('FEED № '.$feedId.'; ERROR: Не удалось запланировань CRON fifty_sec; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
	} else {
		ipytw_error_log('FEED № '.$feedId.'; CRON fifty_sec успешно запланирован; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
	}
 }
 public function ipytw_do_this_fifty_sec($feedId) {
	ipytw_error_log('FEED № '.$feedId.'; Крон ipytw_do_this_fifty_sec запущен; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
	$time_start_last_step = (int)current_time('timestamp', 1); // unixtime
	$this->ipytw_construct_yml($feedId, $time_start_last_step); // делаем что-либо каждые 50 сек
 }
 /* end функции крона */
 
 // Вывод различных notices
 public function ipytw_admin_notices_function() {
	// нужно ли запускать обновление фида при перезаписи файла
	$ipytw_enable_backend_debug = ipytw_optionGET('ipytw_enable_backend_debug'); 
	if ($ipytw_enable_backend_debug === 'on') {
		print '<div class="updated notice notice-success is-dismissible"><p>';
		print '<h1>Зарегистрированные фиды:</h1>';
		$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
		var_dump($ipytw_registered_feeds_arr);

		print '<h1>Динамические опции:</h1>';
		$ipytw_settings_arr = ipytw_optionGET('ipytw_settings_arr');
		var_dump($ipytw_settings_arr);

		print '<h1>Статусы сборок:</h1>';
		for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
			$feedId = $ipytw_registered_feeds_arr[$i]['id'];
			$ipytw_status_sborki_val = ipytw_optionGET('ipytw_status_sborki', $feedId);
			$ipytw_last_element_val = ipytw_optionGET('ipytw_last_element', $feedId);
			if ($feedId === '0' || $feedId === 0) {$feedId = '';}
			$ipytw_status_sborki_key = 'ipytw_status_sborki'.$feedId;
			$ipytw_last_element_key = 'ipytw_last_element'.$feedId;
			print '$ipytw_status_sborki_key = '.$ipytw_status_sborki_key.'; $ipytw_status_sborki_val = '.$ipytw_status_sborki_val.'<br/>'; 
			print '$ipytw_last_element_key = '.$ipytw_last_element_key.'; $ipytw_last_element_val = '.$ipytw_last_element_val.'<br/>';
		}
		print '<h1>Крон задачи (ipytw_cron_period) и (ipytw_cron_sborki):</h1>';
		$cron_zadachi = get_option('cron'); // получаем все задачи из базы данных
		var_dump($cron_zadachi); // можно использовать функции print_r() или var_dump() для вывода всех задач
		print '</p></div>';
	}
	  
	if (isset($_REQUEST['ipytw_submit_action'])) {
		$run_text = '';
		if (sanitize_text_field($_POST['ipytw_run_cron']) !== 'off') {
			$run_text = '. '. __('Import products is running. You can continue working with the website', 'ipytw');
		}
		print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'ipytw'). $run_text .'.</p></div>';
	}

	if (isset($_REQUEST['ipytw_submit_debug_page'])) {
		print '<div class="updated notice notice-success is-dismissible"><p>'. __('Updated', 'ipytw'). '.</p></div>';
	}

	if (isset($_REQUEST['ipytw_submit_clear_logs'])) {
		$upload_dir = (object)wp_get_upload_dir();
		$name_dir = $upload_dir->basedir."/import-from-yml";
		$filename = $name_dir.'/ipytw.log';
		if (file_exists($filename)) {$res = unlink($filename);} else {$res = false;}
		if ($res == true) {
			print '<div class="notice notice-success is-dismissible"><p>' .__('Logs were cleared', 'ipytw'). '.</p></div>';					
		} else {
			print '<div class="notice notice-warning is-dismissible"><p>' .__('Error accessing log file. The log file may have been deleted previously', 'ipytw'). '.</p></div>';		
		}
	}

	/* сброс настроек */
	if (isset($_REQUEST['ipytw_submit_reset'])) {
		if (!empty($_POST) && check_admin_referer('ipytw_nonce_action_reset', 'ipytw_nonce_field_reset')) {
			$this->on_uninstall();
			$this->on_activation();	
			print '<div class="updated notice notice-success is-dismissible"><p>'. __('The settings have been reset', 'ipytw'). '.</p></div>';
		}
	} /* end сброс настроек */ 

	/* отправка отчёта */
	if (isset($_REQUEST['ipytw_submit_send_stat'])) {
		if (!empty($_POST) && check_admin_referer('ipytw_nonce_action_send_stat', 'ipytw_nonce_field_send_stat')) { 	
		if (is_multisite()) { 
			$ipytw_is_multisite = 'включен';	
			$ipytw_keeplogs = get_blog_option(get_current_blog_id(), 'ipytw_keeplogs');
		} else {
			$ipytw_is_multisite = 'отключен'; 
			$ipytw_keeplogs = get_option('ipytw_keeplogs');
		}
		
		$mail_content = "Версия плагина: ". ipytw_VER . PHP_EOL;
		$mail_content .= "Версия WP: ".get_bloginfo('version'). PHP_EOL;	 
		$woo_version = ipytw_get_woo_version_number();
		$mail_content .= "Версия WC: ".$woo_version. PHP_EOL;
		$mail_content .= "Версия PHP: ".phpversion(). PHP_EOL;   
		$mail_content .= "Режим мультисайта: ".$ipytw_is_multisite. PHP_EOL;
		$mail_content .= "Вести логи: ".$ipytw_keeplogs. PHP_EOL;
		$mail_content .= "Расположение логов: ". ipytw_UPLOAD_DIR .'/import-from-yml/ipytw.log'. PHP_EOL;	
		$possible_problems_arr = ipytw_possible_problems_list();
		if ($possible_problems_arr[1] > 0) {
			$possible_problems_arr[3] = str_replace('<br/>', PHP_EOL, $possible_problems_arr[3]);
			$mail_content .= "Самодиагностика: ". PHP_EOL .$possible_problems_arr[3];
		} else {
			$mail_content .= "Самодиагностика: Функции самодиагностики не выявили потенциальных проблем". PHP_EOL;
		}
		if (!class_exists('ImportProductsYMLtoWooCommercePro')) {
			$mail_content .= "Pro: не активна". PHP_EOL;
		} else {
			if (!defined('ipytwp_VER')) {define('ipytwp_VER', 'н/д');} 
			$order_id = ipytw_optionGET('ipytwp_order_id');
			$order_email = ipytw_optionGET('ipytwp_order_email');			
			$mail_content .= "Pro: активна (v ".ipytwp_VER." (#".$order_id." / ".$order_email."))". PHP_EOL;
		}
		if (isset($_REQUEST['ipytw_its_ok'])) {
			$mail_content .= PHP_EOL ."Помог ли плагин: ".sanitize_text_field($_REQUEST['ipytw_its_ok']);
		}
		if (isset($_POST['ipytw_email'])) {
			$mail_content .= PHP_EOL ."Почта: ".sanitize_email($_POST['ipytw_email']);
		}
		if (isset($_POST['ipytw_message'])) {
			$mail_content .= PHP_EOL ."Сообщение: ".sanitize_text_field($_POST['ipytw_message']). PHP_EOL;
		}

		$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
		for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
			$feedId = $ipytw_registered_feeds_arr[$i]['id'];

			$ipytw_url_yml_file = ipytw_optionGET('ipytw_url_yml_file', $feedId, 'set_arr');
			$ipytw_date_sborki = ipytw_optionGET('ipytw_date_sborki', $feedId, 'set_arr');
			$ipytw_errors = ipytw_optionGET('ipytw_errors', $feedId, 'set_arr');

			$mail_content .= PHP_EOL."ФИД №: ".$feedId. PHP_EOL . PHP_EOL;
			$mail_content .= "УРЛ: ".get_site_url(). PHP_EOL;
			$mail_content .= "УРЛ YML-фида: ".$ipytw_url_yml_file . PHP_EOL;
			$mail_content .= "Дата последнего импорта XML: ".$ipytw_date_sborki. PHP_EOL;
			$mail_content .= "Ошибки: ".$ipytw_errors. PHP_EOL;
		}
		wp_mail('support@icopydoc.ru', 'Отчёт Import from YML', $mail_content);
		print '<div class="updated notice notice-success is-dismissible"><p>'. __('The data has been sent. Thank you', 'ipytw'). '.</p></div>';
		}
	} /* end отправка отчёта */ 

	$ipytw_disable_notices = ipytw_optionGET('ipytw_disable_notices');
	if ($ipytw_disable_notices === 'on') {} else {
		$ipytw_registered_feeds_arr = ipytw_optionGET('ipytw_registered_feeds_arr');
		for ($i = 1; $i < count($ipytw_registered_feeds_arr); $i++) { // с единицы, т.к инфа по конкретным фидам там
			$feedId = $ipytw_registered_feeds_arr[$i]['id'];		
			$status_sborki = ipytw_optionGET('ipytw_status_sborki', $feedId); // ожидается тип (int)
			switch ($status_sborki) {
				case 1:
					print '<div class="updated notice notice-success is-dismissible"><p>'. __('Feed ID', 'ipytw').' = '.$feedId.'. '. __('Import products is running', 'ipytw').'. '. __('Step', 'ipytw').': 1. '. __('Importing a list of categories', 'ipytw').'.</p></div>';
				break;
				case 2:
					$ipytw_last_element = (int)ipytw_optionGET('ipytw_last_element', $feedId) + 1;
					$ipytw_count_elements = ipytw_optionGET('ipytw_count_elements', $feedId, 'set_arr');
					print '<div class="updated notice notice-success is-dismissible"><p>'. __('Feed ID', 'ipytw').' = '.$feedId.'. '. __('Import products is running', 'ipytw').'. '. __('Step', 'ipytw').': 2. '. __('Processed products', 'ipytw').': '.$ipytw_last_element.' '. __('of', 'ipytw').' '.$ipytw_count_elements.'.</p></div>';
				break;
				default:
			}
		}
	}
 }
 
 // сборка
 public static function ipytw_construct_yml($feedId, $time_start_last_step) {
	ipytw_error_log('FEED № '.$feedId.'; Стартовала ipytw_construct_yml. Файл: import-from-yml.php; Строка: '.__LINE__ , 0);

	$status_sborki = (int)ipytw_optionGET('ipytw_status_sborki', $feedId);

	// файл уже собран. На всякий случай отключим крон сборки и обнулис счётчик последнего элемента
	if ($status_sborki == -1 ) {wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId)); ipytw_optionUPD('ipytw_last_element', '-1', $feedId); return;}

	$ipytw_last_element = (int)ipytw_optionGET('ipytw_last_element', $feedId);
	$ipytw_step_import = ipytw_optionGET('ipytw_step_import', $feedId, 'set_arr');
	$timeout = $time_start_last_step + $ipytw_step_import - 2;

	if ($status_sborki == 1) { // начинаем сборку файла
		do_action('ipytw_before_construct', 'full', $feedId); // сборка стартовала
		$result = ipytw_feed_header($feedId);
		if ($result === false) {
			$status_sborki = -1;
			ipytw_optionUPD('ipytw_status_sborki', $status_sborki, $feedId);
			ipytw_optionUPD('ipytw_last_element', '-1', $feedId);
			wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId)); // останавливаем крон сборки
			do_action('ipytw_after_construct', 'full'); // сборка закончена
			ipytw_error_log('FEED № '.$feedId.'; ERROR: Сборка завершена с ошибкой на стадии импорта категорий; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
			return;
		}
		$status_sborki = 2;
		ipytw_optionUPD('ipytw_status_sborki', $status_sborki, $feedId);	
	} 
	if ($status_sborki > 1) { 
		ipytw_error_log('FEED № '.$feedId.'; NOTICE: Сборка на стадии импорта товаров. $status_sborki = '.$status_sborki.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
		$ipytw_url_yml_file = ipytw_optionGET('ipytw_url_yml_file', $feedId, 'set_arr');
		ipytw_error_log('FEED № '.$feedId.'; Читаем файл: '.$ipytw_url_yml_file.'; Файл: offer.php; Строка: '.__LINE__, 0);

		$xml_string = ipytw_file_get_contents_curl($ipytw_url_yml_file);
		if ($xml_string === false) { // нет доступа к серверу
			ipytw_optionUPD('ipytw_status_sborki', '-1', $feedId);
			ipytw_optionUPD('ipytw_last_element', '-1', $feedId);
			wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId));
			do_action('ipytw_after_construct', 'full'); // сборка закончена
			ipytw_error_log('FEED № '.$feedId.'; ERROR: Сборка завершена с ошибкой. Не смогли подключиться к фиду; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
			return;						
		} else {
			$xml_object = new SimpleXMLElement($xml_string);
		}

		$xml_offers_object = $xml_object->shop->offers;

		$feed_imported_cat_arr = ipytw_get_imported_cat($feedId);
		$ipytw_count_elements = ipytw_optionGET('ipytw_count_elements', $feedId, 'set_arr');

		if ($ipytw_count_elements < 0) {
			// число товаров на синхронизацию ещё не определялось. Определим и запишем в БД
			$ipytw_count_elements = count($xml_offers_object->children());
			ipytw_optionUPD('ipytw_count_elements', $ipytw_count_elements, $feedId, 'yes', 'set_arr');
			// также устанавливаем значение счётчик обработанных товаров
			ipytw_optionUPD('ipytw_last_element', 0, $feedId);		
			ipytw_error_log('FEED № '.$feedId.'; Всего товаров в фиде: '.$ipytw_count_elements.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
		}

		if ($ipytw_last_element < $ipytw_count_elements-1) {
			ipytw_error_log('FEED № '.$feedId.'; $ipytw_last_element = '.$ipytw_last_element.' < $ipytw_count_elements = '.$ipytw_count_elements.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
			// ещё не все товары обработаны. Обработаем
			for ($i = $ipytw_last_element; $i < $ipytw_count_elements; $i++) {
				$offer_xml_object = $xml_offers_object->offer[$i];
				$imported_ids = ipytw_unit($offer_xml_object, $feed_imported_cat_arr, $feedId); // $feed_imported_offers_arr 
				if ($imported_ids === false) {
					ipytw_error_log('FEED № '.$feedId.'; ipytw_unit вернула false; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
				} else {
					ipytw_error_log('FEED № '.$feedId.'; ipytw_unit вернула $imported_ids; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
					ipytw_error_log($imported_ids, 0);
				}
				
				// https://wp-kama.ru/handbook/codex/object-cache
				wp_suspend_cache_addition(true);
				$status_sborki = ipytw_optionGET('ipytw_status_sborki', $feedId); // ещё раз проверим необходимость остановки всей сборки
				wp_suspend_cache_addition(false);
				if ($status_sborki == -1) { // останавливаем сборку
					ipytw_optionUPD('ipytw_last_element', '-1', $feedId);	
					wp_clear_scheduled_hook('ipytw_cron_period', array($feedId));
					wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId));
					break;
				}

				ipytw_optionUPD('ipytw_last_element', $i, $feedId);

				$unixtime = (int)current_time('timestamp', 1);
				if ($unixtime >= $timeout) {
					ipytw_error_log('FEED № '.$feedId.'; $unixtime = '.$unixtime.' > $timeout = '.$timeout.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
					break;
				} else {
					ipytw_error_log('FEED № '.$feedId.'; $unixtime = '.$unixtime.' < $timeout = '.$timeout.'; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
				}
			}
		} else {
			ipytw_optionUPD('ipytw_status_sborki', '-1', $feedId);
			ipytw_optionUPD('ipytw_last_element', '-1', $feedId);
			ipytw_optionUPD('ipytw_count_elements', $ipytw_count_elements, $feedId, -1, 'set_arr');
			// останавливаем крон сборки
			wp_clear_scheduled_hook('ipytw_cron_sborki', array($feedId));
			do_action('ipytw_after_construct', 'full'); // сборка закончена
			ipytw_error_log('FEED № '.$feedId.'; SUCCESS: Сборка успешно завершена; Файл: import-from-yml.php; Строка: '.__LINE__, 0);
		}
		return;
	} // end if ($status_sborki > 1)
	return;
 } // end public static function ipytw_construct_yml
} /* end class ImportProductsYMLtoWooCommerce */
?>